;this demo is compiled with asm6


;iNES header

	.db "NES",$1a
	.db 2			;2 16K PRG
	.db 1			;1 8K CHR
	.db 0			;mapper 0, horizontal mirroring
	.dsb 9,0		;other header bytes



;register aliases

PPU_CTRL		equ $2000
PPU_MASK		equ $2001
PPU_STATUS		equ $2002
PPU_OAM_ADDR	equ $2003
PPU_SCROLL		equ $2005
PPU_ADDR		equ $2006
PPU_DATA		equ $2007
PPU_OAM_DMA		equ $4014
PPU_FRAMECNT	equ $4017
DMC_FREQ		equ $4010
CTRL_PORT1		equ $4016

;gamepad readback bits aliases

PAD_A			equ $01
PAD_B			equ $02
PAD_SELECT		equ $04
PAD_START		equ $08
PAD_UP			equ $10
PAD_DOWN		equ $20
PAD_LEFT		equ $40
PAD_RIGHT		equ $80



;demo variables

	.enum $0080

FRAME_CNT		.dsb 1
PTR				.dsb 2

	.ende





;demo code

    .org $8000

reset:

;init hardware

    sei
    ldx #$40
    stx PPU_FRAMECNT
    ldx #$ff
    txs
    inx
    stx PPU_MASK
    stx DMC_FREQ
	lda #%10000000
    sta PPU_CTRL

	jsr waitVBlank

    txa
	
clearRAM:

    sta $000,x
    sta $100,x
    sta $200,x
    sta $300,x
    sta $400,x
    sta $500,x
    sta $600,x
    sta $700,x
    inx
    bne clearRAM

	jsr waitVBlank
	
	lda #$00
	sta PPU_SCROLL
	sta PPU_SCROLL

clearVRAM:

	lda #$20
	ldy #$00
	sta PPU_ADDR
	sty PPU_ADDR
	
	ldx #$08

@loop:

	sta PPU_DATA
	iny
	bne @loop
	dex
	bne @loop

	stx FRAME_CNT



copyNametables

	lda #$20
	sta PPU_ADDR
	lda #$00
	sta PPU_ADDR

	ldx #<screen_a_nam
	ldy #>screen_a_nam
	
	jsr copyNametable

	lda #$28
	sta PPU_ADDR
	lda #$00
	sta PPU_ADDR
	
	ldx #<screen_b_nam
	ldy #>screen_b_nam
	
	jsr copyNametable
	
	lda #0
	sta PPU_ADDR
	sta PPU_ADDR

	jsr waitNMI

	lda #%00001110	;enable display, background only
	sta PPU_MASK

	
	

mainLoop:

	jsr waitNMI
	
	lda #0
	sta PPU_OAM_ADDR
	lda #screen_a_oam/256
	sta PPU_OAM_DMA
	
	lda #$3f
	sta PPU_ADDR
	lda #$00
	sta PPU_ADDR
	ldx #0
	
@set_palette_a:

	lda screen_a_pal,x
	sta PPU_DATA
	inx
	cpx #32
	bne @set_palette_a
	
	lda #0
	sta PPU_ADDR
	sta PPU_ADDR
	
	lda #%10000000
	sta PPU_CTRL
	
	
	
	jsr waitNMI
	
	lda #0
	sta PPU_OAM_ADDR
	lda #screen_b_oam/256
	sta PPU_OAM_DMA
	
	lda #$3f
	sta PPU_ADDR
	lda #$00
	sta PPU_ADDR
	ldx #0
	
@set_palette_b:

	lda screen_b_pal,x
	sta PPU_DATA
	inx
	cpx #32
	bne @set_palette_b
	
	lda #0
	sta PPU_ADDR
	sta PPU_ADDR
	
	lda #%10011010
	sta PPU_CTRL
	
	
	
	jmp mainLoop




copyNametable

	stx PTR+0
	sty PTR+1

	ldy #0
	ldx #4
	
@loop

	lda (PTR),y
	sta PPU_DATA
	iny
	bne @loop
	inc PTR+1
	dex
	bne @loop
	
	rts
	
	
	
;wait VBlank using PPU status

waitVBlank:

    bit PPU_STATUS
	
@loop:

    bit PPU_STATUS
    bpl @loop
	
	rts


	
;wait NMI and VBlank using counter in the NMI handler

waitNMI:

	lda FRAME_CNT
	
@loop:

	cmp FRAME_CNT
	beq @loop
	
	rts


	
;NMI handler

nmi:

	inc FRAME_CNT
	
irq:

    rti



	.align 256
	
screen_a_oam:		.incbin "screen_a.oam"
screen_b_oam:		.incbin "screen_b.oam"

screen_a_nam:		.incbin "screen_a.nam"
screen_b_nam:		.incbin "screen_b.nam"

screen_a_pal:		.incbin "screen_a.pal"
screen_a_spr_pal:	.incbin "screen_a_spr.pal"

screen_b_pal:		.incbin "screen_b.pal"
screen_b_spr_pal:	.incbin "screen_b_spr.pal"

;vectors

    .org  $fffa
    .dw nmi
    .dw reset
	.dw irq
	

;CHR

	.incbin "screen_a.chr"
	.incbin "screen_b.chr"